<?php

function node_gallery_api_get_relationships($gallery_nid = NULL, $item_nid = NULL, $id = NULL) {
  $query = new EntityFieldQuery();
  $query->entityCondition('entity_type', 'node_gallery_relationship');
  if(!empty($item_nid)) {
    $query->propertyCondition('nid', $item_nid);
  }
  if(!empty($gallery_nid)) {
    $query->propertyCondition('ngid', $gallery_nid);
  }
  if(!empty($id)) {
    $query->entityCondition('id', $id);
  }
  $relationships = $query->execute();
  if(!empty($relationships['node_gallery_relationship'])) {
    $relationships = array_keys($relationships['node_gallery_relationship']);
    return entity_load('node_gallery_relationship', $relationships);
  }
  return array();
}


/**
 * Convenience wrapper for getting all relationships.
 * @param $reset
 *
 * @return
 *   An associative array of relationship arrays.
 */
function node_gallery_api_get_all_relationship_types() {
  return node_gallery_api_get_relationship_type(NULL, NULL);
}



/**
 * Fetches a gallery-to-image relationship from the database.
 *
 * If one of either $gallery_type or $item_type are supplied, this function
 * returns a single relationship.  If neither are specified, it returns all
 * relationships, keyed by the gallery content type.  If both are supplied,
 * $item_type will be ignored.
 *
 * @param $gallery_type
 *   (optional) The content type of the gallery in the relationship.  If
 *   this is an int, we'll use that as the rid.
 *   Defaults to NULL.
 * @param $item_type
 *   (optional) The content type of the image in the relationship.
 *   Defaults to NULL.
 *
 * @return
 *   An associative array containing:
 *   - rid: An integer representing the rid column in the database.
 *   - gallery_type: A string that is the content type of the gallery.
 *   - item_type: A string that is the content type of the image.
 *   - filefield_name: A string that is the name of the imagefield for the image.
 *   - settings: An associative array of settings for the relationship.
 */
function node_gallery_api_get_relationship_type($gallery_type = NULL, $item_type = NULL, $id = NULL, $reset = FALSE) {
  $relationship_types = &drupal_static(__FUNCTION__);

  if(!isset($relationship_types) || $reset) {
    $query = new EntityFieldQuery();
    $query->entityCondition('entity_type', 'node_gallery_relationship_type');
    $entities = $query->execute();
    if(!empty($entities['node_gallery_relationship_type'])) {
      $relationship_types = entity_load('node_gallery_relationship_type', array_keys($entities['node_gallery_relationship_type']));
    }
  }

  if(empty($relationship_types)) {
    return NULL;
  }
  else if(empty($gallery_type) && empty($item_type) && empty($id)) {
    return $relationship_types;
  }

  $filtered_types = $relationship_types;
  foreach($filtered_types as $i => $type) {
    if(!empty($id) && $i != $id) {
      unset($filtered_types[$i]);
    }
    if(!empty($gallery_type) && !in_array($gallery_type, $type->settings['relationship_type']['gallery_types'])) {
      unset($filtered_types[$i]);
    }
    if(!empty($item_type) && !in_array($item_type, $type->settings['relationship_type']['item_types'])) {
      unset($filtered_types[$i]);
    }
  }

  if(empty($filtered_types)) {
    return NULL;
  }
  else {
    return array_shift($filtered_types);
  }

  /*
  $return_all = empty($gallery_type) && empty($item_type) && empty($id);

  if($return_all) {
    $query = new EntityFieldQuery();
    $query->entityCondition('entity_type', 'node_gallery_relationship_type');
    $entities = $query->execute();
    if(!empty($entities['node_gallery_relationship_type'])) {
      $possible_ids = array_keys($entities['node_gallery_relationship_type']);
    }
  }
  else if(!empty($id) && empty($item_type)) {
    $possible_ids = array($id);
  }
  else {
    // Search for possible matches on item_type
    $query = db_select('field_config_instance', 'fci')
              ->fields('fci', array('field_name'));
    if(empty($id)) {
      $query->condition('field_name', NODE_GALLERY_REF_FIELD . '_%', 'LIKE');
    }
    else {
      $query->condition('field_name', NODE_GALLERY_REF_FIELD . '_' . $id);
    }
    if(!empty($item_type)) {
      $query->condition('bundle', $item_type);
    }
    $fields = $query->execute();

    $possible_ids = array();
    foreach($fields as $field) {
      if(preg_match('/' . NODE_GALLERY_REF_FIELD . '_(\d+)/', $field->field_name, $matches)) {
        $possible_ids[] = $matches[1];
      }
    }
  }

  if(empty($possible_ids)) {
    return NULL;
  }

  // Now we can verify the gallery_type
  if(!empty($gallery_type)) {
    foreach($possible_ids as $i => $rel_id) {
      $field_info = field_info_field(NODE_GALLERY_REF_FIELD . '_' . $rel_id);
      if(!in_array($gallery_type, $field_info['settings']['handler_settings']['target_bundles'])) {
        unset($possible_ids[$i]);
      }
    }
  }

  if(!empty($possible_ids)) {
    $relationship_types = entity_load('node_gallery_relationship_type', array_values($possible_ids));
  }

  if($return_all) {
    return $relationship_types;
  }
  else if(is_array($relationship_types) && !empty($relationship_types)) {
    return array_shift($relationship_types);
  }
  return NULL;
  */
}

/**
 * Gets the count of published images within a gallery.
 * @param $gid
 *   The nid of the gallery to query.
 * @param $reset
 *   (optional) If TRUE, clears the cache, defaults to FALSE.
 *
 * @return
 *   A count of all images in the gallery.
 */
function node_gallery_api_get_item_count($ngid, $reset = FALSE) {
  $item_list = node_gallery_api_get_item_list($ngid, $reset);
  return count($item_list);
}


/**
 * Returns a list of all possible content types of galleries, images, or both.
 *
 * @param $type
 *   (optional) 'gallery', 'image' or 'all'.
 *
 * @return
 *   An array containing the list of content types.
 */
function node_gallery_api_get_types($type = 'gallery', $reset = FALSE) {
  static $ng_types = array();

  if (empty($ng_types) || $reset) {
    $ng_types['gallery'] = array();
    $ng_types['item'] = array();

    $ng_rels = node_gallery_api_get_all_relationship_types();
    if (is_array($ng_rels)) {
      foreach ($ng_rels as  $relationship_type) {
        foreach($relationship_type->gallery_types as $gallery_type) {
          $ng_types['gallery'][] = $gallery_type;
        }
        foreach($relationship_type->item_types as $item_type) {
          $ng_types['item'][] = $item_type;
        }
      }
    }
  }
  if ($type == 'all') {
    return array_merge($ng_types['gallery'], $ng_types['item']);
  }
  else {
    return $ng_types[$type];
  }
}


/**
 * Retrieves image properties sorted by the image sort view in a gallery, the returned objects are NOT complete nodes.
 *
 * @param object $gallery The node object of the gallery.
 * @param boolean $sorted Whether to keep the order as effective in the UI. Defaults to true.
 * @param boolean $filtered Whether to filter down the resulting images by publish status.
 *
 * @return array An array of image objects with the basic properties nid, title, status, created, changed, filepath and weight.
 */
function node_gallery_api_get_items($gallery, $sorted = TRUE, $filtered = TRUE, $reset = FALSE) {
  $items = drupal_static(__FUNCTION__, array(), $reset);
  $cache_keys[] = 'node_gallery';
  $cache_keys[] = $gallery->nid;
  $cache_keys[] = 'items';
  $cache_keys[] = $sorted ? 'sort-true' : 'sort-false';
  $cache_keys[] = $filtered ? 'filtered-true' : 'filtered-false';
  $cache_key = implode(':', $cache_keys);
  if (!isset($images[$cache_key]) || $reset) {
    if (!$reset && ($cache = cache_get($cache_key)) && !empty($cache->data)) {
      return $cache->data;
    }
    else {
      $items[$cache_key] = array();
      // get the order currently in effect; the criteria is unknown here because the user can change the view.
      $item_nids = node_gallery_api_get_item_nids($gallery->nid, $sorted, $filtered);
      if (count($item_nids) == 0) {
        cache_set($cache_key, $items[$cache_key], 'cache', CACHE_TEMPORARY);
        return $items[$cache_key];
      }
      // get CCK information for retrieving the filepath
      $relationship_type = node_gallery_api_get_relationship_type($gallery->type);
      //$files = node_gallery_api_get_item_file($item_nids, $relationship_type->filefield_name);
      //var_dump($files);exit;
    // $field = content_fields($relationship['filefield_name']);
     // $field_db_info = content_database_info($field);
      /*$field_db_table = $field_db_info['table'];
      $field_db_column = $field_db_info['columns']['fid']['column'];

      $sql = 'SELECT n.nid,n.title,n.status,n.created,n.changed,f.filepath,ngi.weight FROM {node} n'
              . ' INNER JOIN {node_revisions} r ON r.vid = n.vid'
              . ' INNER JOIN {' . $field_db_table . '} cck ON cck.nid = n.nid'
              . ' INNER JOIN {files} f ON f.fid = cck.' . $field_db_column
              . ' INNER JOIN {node_gallery_items} ngi ON ngi.nid = n.nid'
              . ' WHERE n.nid IN (' . db_placeholders($image_nids) . ')';
      if ($filtered) {
        $sql .= ' AND n.status = 1';
      }
      $result = db_query($sql, $image_nids);
      */
      foreach ($item_nids as $nid) {
        $item = node_load($nid);
        if(!empty($relationship_type->filefield_name) && !empty($item->{$relationship_type->filefield_name})) {
          $item->item_file_view = file_view((object) $item->{$relationship_type->filefield_name}[LANGUAGE_NONE][0], 'node_gallery_file_thumbnail');
          $item->item_file_url = file_create_url($item->{$relationship_type->filefield_name}[LANGUAGE_NONE][0]['uri']);
        }
        $items[$cache_key][] = $item;
      }
     /* $unordered = array();
      while ($image = db_fetch_object($result)) {
        $unordered[$image->nid] = $image;
      }
      if ($sorted) {
        foreach ($image_nids as $nid) {
          $images[$cache_key][] = $unordered[$nid];
        }
      }
      else {
        $images[$cache_key] = array_values($unordered);
      }*/
      cache_set($cache_key, $items[$cache_key], 'cache', CACHE_TEMPORARY);
    }
  }
  return $items[$cache_key];
}

/**
 * Clears all the caches for a specific gallery.
 *
 * @param $ngid
 *   The nid of the gallery to clear caches on.
 */
function node_gallery_api_clear_gallery_caches($ngid) {
  // watchdog('node_gallery', 'Clearing caches for gallery with node id !gid.', array('!gid' => $ngid), WATCHDOG_INFO, l(t('view gallery'), 'node/'. $ngid));
  cache_clear_all('node_gallery:' . $ngid, 'cache', TRUE);
}


/**
 * @todo Please document this function.
 * @see http://drupal.org/node/1354
 */
function node_gallery_api_batch_sorting_callback($images, &$context) {
  if (empty($context['sandbox'])) {
    $context['sandbox']['iterator'] = 0;
    $context['sandbox']['count'] = count($images);
    $context['results']['ngid'] = $images[0]['ngid'];
  }
  $context['results'][] = drupal_write_record('node_gallery_relationship', $images[$context['sandbox']['iterator']], 'nid');
  $context['sandbox']['iterator']++;
  if ($context['sandbox']['iterator'] < $context['sandbox']['count']) {
    $context['finished'] = $context['sandbox']['iterator'] / (float) $context['sandbox']['count'];
  }
  else {
    $context['finished'] = 1;
  }
}

/**
 * @todo Please document this function.
 * @see http://drupal.org/node/1354
 */
function node_gallery_api_batch_sorting_finished($success, $results, $operation) {
  if ($success) {
    // $results contains ['ngid'] as well as the actual results, so we subtract one here.
    $message = format_plural(count($results) - 1, 'One image processed.', '@count images processed.');
  }
  else {
    $message = t('Finished with an error.');
  }
  node_gallery_api_clear_gallery_caches($results['ngid']);
  drupal_set_message($message);
}


function node_gallery_api_get_item_file($nids, $fieldname) {
  $fields = drupal_static(__FUNCTION__, array());
  static $fields = array();

  if ($fieldname == 'node_gallery_none') {
    return array();
  }

  if (! isset($fields[$fieldname]) ) {
    $field_info = field_info_field($fieldname);
    $fields[$fieldname]['db_info'] = array_shift(array_values($field_info['storage']['details']['sql'][FIELD_LOAD_CURRENT]));
    $fields[$fieldname]['db_info'] += array('table' => array_shift(array_keys($field_info['storage']['details']['sql'][FIELD_LOAD_CURRENT])));
    //$fields[$fieldname]['field'] = content_fields($fieldname);
    //$fields[$fieldname]['db_info'] = content_database_info($fields[$fieldname]['field']);
  }
  //file_load($fid)
  //$table = $fields[$fieldname]['db_info']['table'];
  //$sql = 'SELECT field_table.entity_id AS nid, f.url FROM {' . $table . '} field_table JOIN {files_managed} f ON field_table.' . $fields[$fieldname]['db_info']['fid'] ' = f.fid ';
  $query = db_select($fields[$fieldname]['db_info']['table'], 'field_table');
  //$query->join('file_managed', 'f', 'field_table.' . $fields[$fieldname]['db_info']['fid'] . ' = f.fid');
  $query->fields('field_table', array($fields[$fieldname]['db_info']['fid'], 'entity_id'));
  //$query->fields('f', array('uri'));
  $query->condition('field_table.entity_type', 'node', '=');
  if (!is_array($nids)) {
    // $sql .= ' WHERE nid = %d';
    $query->condition('field_table.entity_id', $nids, '=');
  }
  else {
    $query->condition('field_table.entity_id', $nids, 'IN');
  }
  $results = $query->execute();
  $files = array();
  foreach($results as $record) {
    if(!empty($record->{$fields[$fieldname]['db_info']['fid']})) {
      $files[$record->entity_id] = file_load($record->{$fields[$fieldname]['db_info']['fid']});
    }
  }
  //TODO: Get rid of this inconsistency of use.
  if(count($nids) == 1) {
    $files = array_shift($files);
  }
  return $files;
}


/**
 * @todo Please document this function.
 * @see http://drupal.org/node/1354
 */
function node_gallery_api_batch_node_save($nodes, &$context) {
  //watchdog('test', print_r($nodes, TRUE));
  //watchdog('test', print_r($context, TRUE));
  if (!isset($context['sandbox']['progress'])) {
    $context['sandbox']['progress'] = 0;
    $context['sandbox']['max'] = count($nodes);
  }
  if (!isset($context['sandbox']['nodes'])) {
    if (is_array($nodes)) {
      $context['sandbox']['nodes'] = $nodes;
    }
    else {
      $context['sandbox']['nodes'] = array($nodes);
    }
  }
  $count = 0;
  while (!empty($context['sandbox']['nodes']) && $count <= NODE_GALLERY_BATCH_CHUNK_SIZE) {
    $count++;
    $node = array_shift($context['sandbox']['nodes']);
    $node = (object) array_merge((array) node_load($node->nid), (array) $node);
    watchdog('batch_save', print_r($node, TRUE));
    node_save_action($node);
    watchdog('batch_save', print_r($node, TRUE));
    $context['sandbox']['progress']++;
  }
  // Let the batch engine know how close we are to completion.
  if ($context['sandbox']['progress'] == $context['sandbox']['max']) {
    // Done!
    $context['finished'] = 1;
  }
  else {
    $context['finished'] = $context['sandbox']['progress'] / $context['sandbox']['max'];
  }
}

function node_gallery_api_get_item_field_name($gallery_type = NULL, $item_type = NULL, $relationship_id = NULL) {
  if(empty($relationship_id)) {
    $relationship_type = node_gallery_api_get_relationship_type($gallery_type, $item_type);
    $relationship_id = $relationship_type->id;
  }
  return NODE_GALLERY_REF_FIELD . '_' . $relationship_id;
}

/*function node_gallery_api_save_item(&$node) {
  if(module_exists('filefield_paths')) {
    // TODO: Fix saving issue with filefield_paths.
    // For some reason, we're losing the entity reference when FileField Paths is installed.
    // This isn't the best solution, but it works for now. Got a better one? Please submit a patch.

    // Before we save, let's capture all the current relationships.
    $rels_to_restore = array();
    $gallery_nids = $node->{NODE_GALLERY_REF_FIELD}[LANGUAGE_NONE];
    foreach($gallery_nids as $gallery_nid) {
      $relationships = node_gallery_api_get_relationships($gallery_nid['target_id'], $node->nid);
      watchdog('relationships', print_r($relationships, TRUE));
      foreach($relationships as $r) {
        $rels_to_restore[] = $r;
      }
    }

    // Now save the item
    node_save_action($node);

    // Restore the relationships
     watchdog('test', 'restoring');
    foreach($rels_to_restore as $r) {
      watchdog('test', print_r($r, TRUE));
      unset($r->id);
      $r->save();
    }
    $node = node_load($node->nid, NULL, TRUE);
  }
  else {
    // Just save.
    node_save_action($node);
  }
}*/

/**
 * Deletes batches of images for batch API.
 * _node_gallery_delete
 * @param $imagenids
 *   Array of nids to delete.
 * @param $context
 *   Array provided by batch API.
 */
function node_gallery_api_batch_node_delete($imagenids, &$context) {
  if (!isset($context['sandbox']['progress'])) {
    $context['sandbox']['progress'] = 0;
    $context['sandbox']['max'] = count($imagenids);
  }
  if (!isset($context['sandbox']['imagenids'])) {
    if (is_array($imagenids)) {
      $context['sandbox']['imagenids'] = $imagenids;
    }
    else {
      $context['sandbox']['imagenids'] = array($imagenids);
    }
  }
  $count = 0;
  while (!empty($context['sandbox']['imagenids']) && $count <= NODE_GALLERY_BATCH_CHUNK_SIZE) {
    $count++;
    $nid = array_shift($context['sandbox']['imagenids']);
    node_delete($nid);
    $context['sandbox']['progress']++;
  }
  // Let the batch engine know how close we are to completion.
  if ($context['sandbox']['progress'] == $context['sandbox']['max']) {
    // Done!
    $context['finished'] = 1;
  }
  else {
    $context['finished'] = $context['sandbox']['progress'] / $context['sandbox']['max'];
  }
}

/**
 * @todo Please document this function.
 * @see http://drupal.org/node/1354
 */
function node_gallery_api_batch_rotate($fid, $degrees, &$context) {
  $result = 0;

  $file = file_load($fid);
  $filepath_new = preg_replace_callback(
    '~(?:_r([0-9]+))?(?=(?:\.[^./]*)?$)~',
    create_function('$matches', 'return "_r".(!empty($matches[1]) ? $matches[1]+1 : 1);'),
    $file->uri, 1);
  $new_file = file_move($file, $filepath_new);
  $image = image_load($new_file->uri);
  global $conf;
  $conf['image_jpeg_quality'] = 100;
  if ($image !== FALSE) {
    if ($image->toolkit == 'imagemagick') {
      $result = image_rotate($image, $degrees, 0x000000);
    }
    else {
   //   var_dump($degrees);
      $result = image_rotate($image, $degrees);
    }

    image_save($image);
    $new_file->image_dimensions['width'] = $image->width;
    $new_file->image_dimensions['height'] = $image->height;
    file_save($new_file);
 //   var_dump($result);
 //    var_dump($image);exit;
    image_path_flush($image->source);
//    imagecache_image_flush($filepath);
    // add a _r$i to the filename for cache distinction of rotated images
   /* $filepath_new = preg_replace_callback(
        '~(?:_r([0-9]+))?(?=(?:\.[^./]*)?$)~',
        create_function('$matches', 'return "_r".(!empty($matches[1]) ? $matches[1]+1 : 1);'),
        $filepath, 1);
    $file = (object) field_file_load($filepath);
    $file->timestamp = REQUEST_TIME;
    $file->filepath = $filepath_new;
    $file->filename = basename($filepath_new);
    $filepath_old = $filepath;
    file_move($filepath, $filepath_new);
    drupal_write_record('files', $file, 'fid');*/
  }
  else {
    watchdog('node_gallery', 'Could not open image for rotation');
  }
  $context['results'][] = $result;
}

/**
 * Used as a finished callback for batch API deletion of images.
 *
 * @param $success
 *   Scalar provided by batch API.
 * @param $results
 *   Array provided by batch API.
 * @param $operations
 *   Array provided by batch API.
 */
function node_gallery_api_batch_item_process_finished($success, $results, $operations) {
  if ($success) {
    drupal_set_message(t('Item modifications complete.'));
    cache_clear_all();
   // _field_file_cache(NULL, TRUE);
   // content_clear_type_cache();
  }
  else {
    // An error occurred.
    // $operations contains the operations that remained unprocessed.
    $error_operation = reset($operations);
    $operation = array_shift($error_operation);
    $arguments = array_shift($error_operation);
    $arguments_as_string = implode(', ', $arguments);
    watchdog('node_gallery', "Error when calling operation '%s'('%s')", array($operation, $arguments_as_string));
    drupal_set_message(t('An error occurred and has been recorded in the system log.'), 'error');
  }
}

/**
 * @todo Please document this function.
 * @see http://drupal.org/node/1354
 */
function node_gallery_api_relationship_type_settings_defaults() {
  $settings = array(
    'relationship' => array(
    	'name' => 'Node Gallery Default',
    ),
    'item_view' => array(
      'display_navigator' => 1,
      'view_navigator_item_display' => 'node_gallery_gallery_image_views:page_7',
      'page_fragment' => 0,
    ),
    'manage_items' => array(
      'items_fields' => array('title' => 'title'),
      'items_per_page' => 20,
      'show_gallery_list' => TRUE,
      'enable_rotation' => TRUE,
    ),
  );
  $view_modes = node_gallery_api_get_view_modes();
  foreach ($view_modes as $name => $label) {
    switch($name) {
      /*case 'full':
        $settings['view_modes'][$name]['view'] = 'node_gallery_gallery_image_views:page_1';
        break;
      case 'teaser':
        $settings['view_modes'][$name]['view'] = 'node_gallery_gallery_image_views:page_2';
        break;
      default:
        $settings['view_modes'][$name]['view'] = 'node_gallery_gallery_image_views:page_2';
        break;
*/
    }
  }
  drupal_alter('node_gallery_relationship_default_settings', $settings);
  return $settings;
}

function node_gallery_api_get_view_modes() {
  $entity_info = entity_get_info('node');
  $view_modes = array();
  foreach($entity_info['view modes'] as $name => $info) {
    $view_modes[$name] = $info['label'];
  }
  return $view_modes;
}


/**
 * Gets a sorted list of images nids within a gallery.
 * @param $ngid
 *   The nid of the gallery to query.
 * @param $reset
 *   (optional) If TRUE, clears the cache, defaults to FALSE.
 *
 * @return
 *   An array of sorted image nids with a status of published.
 */
function node_gallery_api_get_item_list($ngid, $reset = FALSE) {
  $item_list = drupal_static(__FUNCTION__, array(), $reset);

  if (!isset($item_list[$ngid]) || $reset) {
    if (!$reset && ($cache = cache_get('node_gallery:' . $ngid . ':item_list'))) {
      $item_list[$ngid] = $cache->data;
    }
    else {
      _node_gallery_api_cache_sorted_item_nids($ngid);
      $cache = cache_get('node_gallery:' . $ngid . ':item_list');
      $item_list[$ngid] = $cache->data;
    }
  }
  return $item_list[$ngid];
}

/**
 * Builds the caches for a gallery and saves them to the db.
 *
 * There are two caches stored for each gallery.  One is a sorted array of nids
 * that represents the items in proper search order.
 * The second is an associative array where the key is the nid of the image
 * and the value is that nid's respective position in the list (1-based).
 *
 * @param $ngid
 *   The nid of the gallery to build caches for.
 */
function _node_gallery_api_cache_sorted_item_nids($ngid) {
  $item_list = node_gallery_api_get_item_nids($ngid, TRUE, TRUE);
  $item_position = array();
  $position = 1;
  foreach ($item_list as $nid) {
    $item_position[$nid] = $position;
    $position++;
  }
  cache_set('node_gallery:' . $ngid . ':item_position', $item_position, 'cache', CACHE_TEMPORARY);
  cache_set('node_gallery:' . $ngid . ':item_list', $item_list, 'cache', CACHE_TEMPORARY);
}

/**
 * Gets the first image nid within a gallery.
 * @param $gid
 *   The nid of the gallery to query.
 * @param $reset
 *   (optional) If TRUE, clears the cache, defaults to FALSE.
 *
 * @return
 *   The nid of the first image in the gallery.
 */
function node_gallery_api_get_first_item($ngid, $reset = FALSE) {
  $item_list = node_gallery_api_get_item_list($ngid, $reset);
  return $item_list[0];
}

/**
 * Gets the last image nid within a gallery.
 * @param $gid
 *   The nid of the gallery to query.
 * @param $reset
 *   (optional) If TRUE, clears the cache, defaults to FALSE.
 *
 * @return
 *   The nid of the last image in the gallery.
 */
function node_gallery_api_get_last_item($ngid, $reset = FALSE) {
  $item_list = node_gallery_api_get_item_list($ngid, $reset);
  return $item_list[count($item_list) - 1];
}

/**
 * Gets the next image in the gallery.
 *
 * @param $gid
 *   The nid of the gallery node.
 * @param $nid
 *   The nid of the image node.
 * @param $reset
 *   (optional)  If TRUE, the caches are cleared.
 *
 * @return
 *   The nid of the next image in the gallery.
 */
function node_gallery_api_get_next_item($ngid, $nid, $reset = FALSE) {
  return node_gallery_api_seek_from_current_item($ngid, $nid, 1, $reset);
}

/**
 * Gets the previous image in the gallery.
 *
 * @param $gid
 *   The nid of the gallery node.
 * @param $nid
 *   The nid of the image node.
 * @param $reset
 *   (optional)  If TRUE, the caches are cleared.
 *
 * @return
 *   The nid of the previous image in the gallery.
 */
function node_gallery_api_get_prev_item($ngid, $nid, $reset = FALSE) {
  return node_gallery_api_seek_from_current_item($ngid, $nid, -1, $reset);
}

/**
 * Returns the nid of the image +/-N steps away from the current image node.
 * @param $gid
 *   The nid of the gallery to query.
 * @param $nid
 *   The nid of the current image.
 * @param signedint $seek
 *   The postive or negative number of slots to seek to.
 * @param $reset
 *   (optional) If TRUE, clears the caches.  Defaults to FALSE.
 */
function node_gallery_api_seek_from_current_item($ngid, $nid, $seek, $reset) {
  $item_list = node_gallery_api_get_item_list($ngid, $reset);
  $current_position = node_gallery_api_get_item_position($ngid, $nid, $reset);
  $current_index = $current_position - 1;
  $seek_index = $current_index + $seek;

  return isset($item_list[$seek_index]) ? $item_list[$seek_index] : NULL;
}

/**
 * Publishes the child image nodes when publishing a gallery.
 *
 * When publishing a gallery, we publish the image nodes within that gallery.
 * We use batch api to provide for galleries with a large amount of images.
 *
 * @param object $node
 *   A reference to a gallery node object
 * @param $status
 *   The new publish status to set on the node
 */
function _node_gallery_api_set_publish(&$node, $status) {
  $ngid = $node->nid;
  $imagenids = node_gallery_api_get_item_nids($ngid, FALSE, FALSE);
  // Split our operations into NODE_GALLERY_BATCH_CHUNK_SIZE a time
  $node_updates = array_chunk($imagenids, NODE_GALLERY_BATCH_CHUNK_SIZE);
  array_walk_recursive($node_updates,
            create_function('&$v,$k,$status', '$v = (object)array(\'nid\' => $v, \'status\' => $status);'), $status);
  foreach ($node_updates as $node_update) {
    $operations[] = array('node_gallery_api_batch_node_save', array($node_update));
  }
  if (!empty($operations)) {
    $batch = array(
      'operations' => $operations,
      'finished' => 'node_gallery_api_item_process_finished',
      'title' => t('Processing gallery publish status update.'),
      'init_message' => t('Gallery publish status is cascading to images.'),
      'progress_message' => t('Processing publishing of images.'),
      'error_message' => t('Image publish status change has encountered an error.'),
    );
    batch_set($batch);
    node_gallery_api_clear_gallery_caches($ngid);
  }
}

/**
 * @todo Please document this function.
 * @see http://drupal.org/node/1354
 */
function node_gallery_api_get_item_nids($ngid, $sorted = TRUE, $filtered = TRUE, $fallback = FALSE, $reset = FALSE) {
  static $nids = array();
  $cache_keys[] = 'node_gallery';
  $cache_keys[] = $ngid;
  $cache_keys[] = 'item_nids';
  $cache_keys[] = $sorted ? 'sort-true' : 'sort-false';
  $cache_keys[] = $filtered ? 'filtered-true' : 'filtered-false';
  $cache_keys[] = $fallback ? 'fallback-true' : 'fallback-false';
  $cache_key = implode(':', $cache_keys);
  if (!isset($nids[$cache_key]) || $reset) {
    if (!$reset && ($cache = cache_get($cache_key)) && !empty($cache->data)) {
      $nids[$cache_key] = $cache->data;
    }
    else {
      if (!$fallback) {
        $gallery = node_load($ngid);
        $relationship_type = node_gallery_api_get_relationship_type($gallery->type);
        list($view_name, $view_display) = explode(':', $relationship_type->settings['item_view']['view_navigator_item_display']);
        // TODO: Fix views-based sorting.
        unset($view_name);
        if (isset($view_name)) {
          $view = views_get_view($view_name);
          if (!isset($view->display[$view_display])) {
            $message = t('Unable to find the sort view specified in the !relationship, maybe you need to !revert your view?',
              array(
              '!revert' => l(t('revert'), 'admin/structure/views/delete/' . $view_name),
              '!relationship' => l(t('relationship settings'), 'admin/config/content/node-gallery/relationship-type/settings/' . $relationship_type->id),
            ));
          }
          else {
            $view->set_display($view_display);
            $view->set_arguments(array($gallery->nid));
            if (!$sorted) {
              $sorts = $view->get_items('sort', $view_display);
              foreach ($sorts as $sort) {
                $view->set_item($view_display, 'sort', $sort['id'], NULL);
              }
            }
            if (!$filtered) {
              $filters = $view->get_items('filter', $view_display);
              foreach ($filters as $filter) {
                $view->set_item($view_display, 'filter', $filter['id'], NULL);
              }
            }
            $view->build();
            if (isset($view->query->fields['nid'])) {
              $query = $view->build_info['query'];
            }
            else {
              $message = t('Found the image sort view, but the \'Node: Nid\' field wasn\'t found.', array());
            }
          }
        }
        else {
          $message = t('Unable to find a navigator image sort view.  Please revisit your !rellink.', array('!rellink' => l(t('Node Gallery relationship settings'), 'admin/config/content/node_gallery/relationship/settings/' . $relationship_type->id)));
        }
      }
      if (isset($message) || $fallback) {
        $query = db_select('node_gallery_relationship', 'ng')->fields('ng', array('nid'));
        $query->join('node', 'n', 'ng.nid = n.nid');
        // Use our default sql
        if (isset($message)) {
          if (user_access('administer node gallery')) {
            drupal_set_message($message, 'warning');
          }
          watchdog('node_gallery', $message, NULL, WATCHDOG_WARNING);
        }
        if ($filtered) {
          $query->condition('n.status', 1);
        }
        if ($sorted) {
          $query->orderBy('ng.weight', 'ASC');
          $query->orderBy('n.nid', 'ASC');
        }
        $query->condition('ng.ngid', $ngid);
      }
      $result = $query->execute();
      $nids[$cache_key] = array();
      foreach ($result as $r) {
        $nids[$cache_key][] = $r->nid;
      }
      cache_set($cache_key, $nids[$cache_key], 'cache', CACHE_TEMPORARY);
    }
  }
  return $nids[$cache_key];
}

/**
 * Returns the position (starting at one) of the image in the gallery list.
 *
 * @param $ngid
 *   The nid of the gallery to use.
 * @param $nid
 *   The nid of the image to return the position of.
 *
 * @return
 *   The position of the image in the list of published images in the gallery.
 */
function node_gallery_api_get_item_position($ngid, $nid, $reset = FALSE) {
  static $item_position = array();

  if (!isset($item_position[$ngid]) || $reset) {
    if (!$reset && ($cache = cache_get('node_gallery:' . $ngid . ':item_position')) && !empty($cache->data)) {
      $item_position[$ngid] = $cache->data;
    }
    else {
      _node_gallery_api_cache_sorted_item_nids($ngid);
      $cache =  cache_get('node_gallery:' . $ngid . ':item_position');
      $item_position[$ngid] = $cache->data;
    }
  }
  return $item_position[$ngid][$nid];
}



/**
 * Given a gallery, returns it's cover image.
 *
 * @param object $gallery
 *   A populated node object.
 *
 * @return
 *   The nid of the cover image.
 */
function node_gallery_api_get_cover_nid($ngid) {
  return db_query("SELECT cover_item FROM {node_gallery_galleries} WHERE ngid = :ngid", array(':ngid' => $ngid))->fetchField();
}

/**
 * Sets the cover image in the DB if necessary.
 *
 * @param $image
 *   A reference to the node object of a node gallery image.
 */
function node_gallery_api_set_gallery_cover_item($item_nid, $ngid = NULL) {
  if(empty($ngid)) {
    $relationships = node_gallery_api_get_relationships(NULL, $item_nid);
    if(!empty($relationships)) {
      $relationship = array_shift($relationships);
      $ngid = $relationships->ngid;
    }
    if(empty($ngid)) {
      return FALSE;
    }
  }
  db_update('node_gallery_galleries')
    ->fields(array(
      	'cover_item' => $item_nid,
      ))
      ->condition('ngid', $ngid)
      ->execute();

  return TRUE;

  /*if(!empty($ngid)) {
    $image->ngid = $ngid;
  }
  if ($image->ngid > 0) {
    $gallery = new stdClass;
    $gallery->ngid = $image->ngid;
    if ($image->is_cover) {
      if (!empty($image->oldgid)) {
        // this image is the cover image for another gallery, it's being moved. Retrieve a new cover for the old gallery
        node_gallery_api_reset_cover_item($image->oldgid);
      }
      else {
        $gallery->cover_item = $image->nid;
      }
    }
    // check if the gallery already has a cover
    $sql = "SELECT COUNT(*) from {node_gallery_galleries} where ngid = :ngid and cover_item IS NULL";
    $count = db_query($sql, array(':ngid' => $image->ngid))->fetchField();
    if ($count > 0) {
      // This is the first image being added to the gallery.  Make it the cover.
      $gallery->cover_item = $image->nid;
    }
    if (isset($gallery->cover_item)) {
      drupal_write_record('node_gallery_galleries', $gallery, 'ngid');
    }
  }*/
}

function node_gallery_api_reset_cover_item($ngid) {
  $new_cover = db_query('SELECT nid FROM {node_gallery_relationship} WHERE ngid = :ngid LIMIT 1',
                array(':ngid' => $ngid))->fetchField();
  if ($new_cover !== FALSE) {
    $result = db_update('node_gallery_galleries')
      ->fields(array(
      	'cover_item' => $new_cover,
      ))
      ->condition('ngid', $ngid)
      ->execute();
  }
  else {
    // Gallery is now empty, set cover to NULL.
    db_update('node_gallery_galleries')
      ->fields(array(
      	'cover_item' => NULL,
      ))
      ->condition('ngid', $ngid)
      ->execute();
  }
}

function node_gallery_api_get_gallery_ref_fields() {
  $gallery_ref_fields = &drupal_static(__FUNCTION__);

  if (!isset($gallery_ref_fields)) {
    $gallery_ref_fields = array();
    $fields = field_info_fields();
    foreach ($fields as $field_name => $field_info) {
      if ($field_info['type'] == 'entityreference' && !empty($field_info['settings']['handler_settings']['behaviors']['node_gallery_behavior']['status'])) {
        $gallery_ref_fields[$field_name] = $field_info;
      }
    }
  }
  return $gallery_ref_fields;
}


/**
 * Check if we need to update this image;
 *
 * @param unknown_type $old_image
 * @param unknown_type $new_image
 * @param unknown_type $compare_fields
 * @return unknown
 */
function node_gallery_api_items_check_update($old_image, $_new_image, $compare_fields) {
  // a list of node attributes that are flattened one level by node_save
  $flattened_attributes = array('options');

  $new_image = clone $_new_image;
  $new_image = node_submit($new_image);

  if ($extra = node_invoke($new_image, 'presave')) {
    foreach ($extra as $key => $value) {
      $new_image->$key = $value;
    }
  }
  if ($extra = module_invoke_all('node_presave', $new_image)) {
    foreach ($extra as $key => $value) {
      $new_image->$key = $value;
    }
  }
  if ($extra = node_invoke($old_image, 'presave')) {
    foreach ($extra as $key => $value) {
      $old_image->$key = $value;
    }
  }
  if ($extra = module_invoke_all('node_presave', $old_image)) {
    foreach ($extra as $key => $value) {
      $old_image->$key = $value;
    }
  }

  foreach ($compare_fields as $f) {
    if (in_array($f, $flattened_attributes)) {
      // We make the old node look like the new one, ie $old_node->options['status'] = $old_node->status
      foreach (array_keys($new_image->{$f}) as $option) {
        $old_image->{$f}[$option] = $old_image->$option;
      }
    }
    //a hack for cck field validate;
    if (is_array($new_image->{$f})) {
      foreach ($new_image->{$f} as &$ff) {
        if (is_array($ff)) {
          unset($ff['_error_element']);
        }
      }
    }
    // special case for taxonomy, as its representation in form_state is different than in the node object.
    if ($f == 'taxonomy') {
      if (node_gallery_api_taxonomy_compare($old_image, $new_image)) {
        return TRUE;
      }
      else {
        continue;
      }
    }
    if ($new_image->{$f} != $old_image->{$f}) {
      return TRUE;
    }
  }
  return FALSE;
}


/**
 * Compare taxonomy on an unsaved node object versus a node_load()ed object
 * // TODO: This function almost certainly does not work with the new term fields.
 *
 * @param $old
 *   The old node_load()ed object.
 * @param $new
 *   The new node object.
 *
 * @return
 *   TRUE if the taxo differs, FALSE if not
 */
function node_gallery_api_taxonomy_compare($old, $new) {
  $new->taxonomy = taxonomy_preview_terms($new);
  $new_taxonomy = $new->taxonomy;
  $old_taxonomy = $old->taxonomy;

  if (isset($new_taxonomy['tags'])) {
    // in case of free tagging, the preview function does not return term objects
    $existing_tags = array();
    $term2tid = array();
    foreach ($old_taxonomy as $term) {
      $existing_tags[$term->vid][] = $term->name;
      $term2tid[$term->vid][$term->name] = $term->tid;
    }
    foreach ($new_taxonomy['tags'] as $vid => $tag_string) {
      $tags = drupal_explode_tags($tag_string);
      if (count($tags) != count($existing_tags[$vid])) {
        return TRUE;
      }
      foreach ($tags as $tag) {
        if (!in_array($tag, $existing_tags[$vid])) {
          return TRUE;
        }
        else {
          unset($old_taxonomy[$term2tid[$vid][$tag]]);
        }
      }
    }
    unset($new_taxonomy['tags']);
  }
  if ($new_taxonomy != $old_taxonomy) {
    // difference in non-tag vocab
    return TRUE;
  }
  return FALSE;
}


/**
 * Update image counts related to the Node Gallery node.
 *
 * @param $node a gallery or an image node object
 */
function node_gallery_api_update_image_counts($ngid) {
  $res = db_query("SELECT n.status AS status, COUNT(*) AS num
     FROM {node_gallery_relationship} ngi
     INNER JOIN {node} n ON ngi.nid = n.nid
     WHERE ngi.ngid = :ngid GROUP BY n.status", array(':ngid' => $ngid));
  $counts = array(
    0 => 0,
    1 => 0,
  );
  foreach ($res as $row) {
    $counts[(int) $row->status] = (int) $row->num;
  }
  // TODO Please review the conversion of this statement to the D7 database API syntax.
  db_update('node_gallery_galleries')
  ->fields(array(
    'item_count' => $counts[0] + $counts[1],
    'pub_item_count' => $counts[1],
  ))
  ->condition('ngid', $ngid)
  ->execute();
}

/**
 * Returns an array of galleries, suitable for use in a form select.
 *
 * @param $type
 *   The content type of the galleries to be queried.
 * @param $uid
 *   (optional) if specified, the list returned will only contain galleries the user can modify.
 *
 * @return
 *   Associative array where the keys are the nid, and the value is the node title.
 */
function node_gallery_api_get_gallery_list($type, $uid = NULL) {
  $query = db_select('node', 'n')->fields('n', array('nid', 'title'))->condition('type', $type);
  $items = array();
  if ($uid) {
    $query->condition('uid', $uid);
  }
  $result = $query->execute();
  foreach ($result as $r) {
    $items[$r->nid] = $r->title;
  }
  return $items;
}